library(loo)
library(coda)
library(rstan)
source("plotPostPredField.r")

rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

#------------------
#load data from file
shopData=read.csv(file="rawDataField.csv") #participants estimates
load("bonList.RData") #a list with receipts that contain the actual prices of each item and the respective category number

attach(shopData)   


#analysis outside of Stan
mSumFirst      = function (bonList, exponent){ exponent[1]*sapply(bonList, sum)^exponent[2] }

mExponentFirst = function (bonList, exponent){ sapply(bonList, function(vec) {sum(exponent[1]*vec^exponent[2])})  }   


#quadratic loss functions
loss.ExponentFirst = function (exponent, bonList, estimate){
  
   prediction=mExponentFirst(bonList, exponent[1:2])
   
   #log-log-likelihood (to account for heteroscedascidity)
   #ll=dnorm(log(estimate), mean=log(prediction), sd=exponent[3])
   #loss=-2*sum(log(ll))
   
   
   #likelihood
   ll=dnorm(estimate, mean=prediction, sd=exponent[3])
   loss=-2*sum(log(ll))
   
   return(loss)
}


loss.SumFirst = function (exponent, bonList, estimate){
   prediction=mSumFirst(bonList, exponent[1:2])
   
   #log-log-likelihood (to account for heteroscedascidity)
   #ll=dnorm(log(estimate), mean=log(prediction), sd=exponent[3])
   #loss=-2*sum(log(ll))
   
   #log-likelihood
   ll=dnorm(estimate, mean=prediction, sd=exponent[3])
   loss=-2*sum(log(ll))
   
   return(loss)
}


fit.ExponentFirst= nlminb(start=c(1,1,20), objective=loss.ExponentFirst, bonList=bonList, estimate=Schaetzung, lower=c(0.0001,0.0001, 0.0001))
fit.SumFirst     = nlminb(start=c(1,1,20), objective=loss.SumFirst, bonList=bonList, estimate=Schaetzung, lower=c(0.0001,0.0001, 0.0001))

#better fit? (smaller loss = better fit)
g2=fit.SumFirst$objective-fit.ExponentFirst$objective

#BIC
k = length(fit.SumFirst$par) #number of parameters (same for both models)
n = length(bonList) #number of observations

BIC.SumFirst      = fit.SumFirst$objective+k*log(n)
BIC.ExponentFirst = fit.ExponentFirst$objective+k*log(n)

BF = exp((BIC.ExponentFirst-BIC.SumFirst)/2) #values >1 = evidence for Sum First
BF

detach(shopData)


#---------------
# Same analysis in Stan

   #number of samples
   iter   = 12000
   chains = 2
   warmup = 2000

   n=length(bonList)

   #prepare the data for STAN in long format
   itemVec  = NULL
   guessVec = NULL
   nItems   = NULL
   
   for (i in 1:n) {
   
     this.items=bonList[[i]] 
     this.Guess=shopData$Schaetzung[i]  
     
     itemVec =c(itemVec, this.items)
     guessVec=c(guessVec, this.Guess)
     nItems  =c(nItems, length(this.items))
   }  
   
   upperIdx=cumsum(nItems)
   lowerIdx=(upperIdx-nItems)+1



#=============
# Sum first, exponent second

   #datalist
   stanData =list( 
      N           = n, #number of participants
      nTrials     = length(itemVec),
      upperIdx    = upperIdx,
      basketSize  = nItems,
      itemVec     = itemVec,
      guessVec    = guessVec
   )

   initFunc <-function (i) {
      initList=list()
      for (ll in 1:i){
         initList[[ll]]= list (
            
            b1       = 1,
            exponent = 1,
            std      = 1
         )
      }
      return(initList)
   }

   #send data to stan
   fitSumFirst <- stan(file   = 'fieldStudySumFirst.stan', 
                       data   = stanData, 
                       iter   = iter, 
                       chains = chains, 
                       warmup = warmup, 
                       init   = initFunc(2), 
                       thin   = 1)
   
   print(fitSumFirst,pars=c("b1", "exponent", "std"))
   plot(fitSumFirst,pars=c("b1", "exponent", "std"))
   pairs(fitSumFirst, pars = c("b1", "exponent"))
   traceplot(fitSumFirst,pars=c("b1", "exponent","std") )


#=============
# Power first, sum second
   
   fitPowerFirst <- stan(file   = 'fieldStudyPowerFirst.stan', 
                         data   = stanData, 
                         iter   = iter, 
                         chains = chains, 
                         warmup = warmup, 
                         init   = initFunc(2), 
                         thin   = 1)
   
   print(fitPowerFirst,pars=c("b1", "exponent", "std"))
   plot(fitPowerFirst,pars=c("b1", "exponent", "std"))
   pairs(fitPowerFirst, pars = c("b1", "exponent"))
   traceplot(fitPowerFirst,pars=c("b1", "exponent","std") )

  
#-------------
#Compare models

   returnLoo  = function (fit) loo(extract_log_lik(fit)) 
   returnWAIC = function (fit) waic(extract_log_lik(fit)) 
   
   #===== LOO
   loo.PowerFirst = returnLoo(fitPowerFirst)
   loo.SumFirst   = returnLoo(fitSumFirst)
 
   #looic & waic: smaller = better 
   compare(loo.PowerFirst, loo.SumFirst)
  
   
   #===== WAIC
   waic.PowerFirst = returnWAIC(fitPowerFirst)
   waic.SumFirst   = returnWAIC(fitSumFirst)
   
   compare(waic.PowerFirst, waic.SumFirst)
   
   
   #compare log-likelihoods
   
   returnLL = function(fit){
     
     ll=extract(fit, "log_lik")$log_lik
     meanLL=mean(ll)
     maxLL=max(rowSums(ll)) #chain that generated the highest (log)likelihood
     maxLLidx=which(maxLL==rowSums(ll))[1] 
     return(c(mean=meanLL, max=maxLL, idx=maxLLidx))
   }
   
   
   ll=rbind( ll.Pfirst.noWeight        =returnLL(fitPowerFirst),
             ll.Sfirst.noWeight        =returnLL(fitSumFirst)
            )
   
   #display log-likelihood in decreasing order (best model on top)
   ll[order(ll[,"mean"],decreasing=T ),]
   

   
#plot posterior predictive fit
   

   pdf(file="posteriorPredictive.Fieldstudy.pdf", width=8.5, height=11) #create pdf in letter format
   par(mfrow=c(2,1))
   plotPostPredField(observed=guessVec,fitPowerFirst, modelname="Model: power first")
   plotPostPredField(observed=guessVec,fitSumFirst, modelname="Model: sum first")
   dev.off()
   