#Analysis of the field study

library(moments) #to calculate skewness and kurtosis
library(BayesFactor)


#--------------------
#some handy functions

   bootstrapCI = function (vec, conf=.95, nSamples=30000 ) {
      # calculate bootstrapped CI of vec 
      # nSamples = number of bootstrap samples to be drawn
      # conf= 95% confidence interval (2-sided)
      
      twoSided=(1-conf)/2
      bootMean=sapply(1:nSamples, function(i) { mean(sample(vec, replace = TRUE))})
      ci=quantile(bootMean,probs=c(twoSided,1-twoSided)) 
      return(ci=ci)  
   }
   
   
   se = function (vec) {
      #calculate the standard error of the mean
      stdev=sd(vec,na.rm=T)
      n=sum(!is.na(vec))
      return(stdev/sqrt(n))
   }


   ci = function (vec, p=.95) {
      #calculate confidence interval of the mean. Defaults to 95% CI
      se=se(vec)
      this.p=(1-p)/2
      zCrit=qnorm(1-this.p)
      mean.vec=mean(vec, na.rm=T)
      lower=mean.vec-zCrit*se
      upper=mean.vec+zCrit*se
      
      return(c(lowerCI=lower, upperCI=upper))
   }

   addErrorbars = function (xPos,ci) {
      #adds errorbars to barplots
      nBars=length(ci)
      for (bar in 1:nBars) {
         arrows(x0=xPos[bar], y0=ci[[bar]]["lowerCI"],x1=xPos[bar], y1=ci[[bar]]["upperCI"] , angle=90,length=.05, code=3)
      }
   }

   getBF=function(lm1, lm2) {
      #returns BF for two linear models
      #Positive BF = evidence for lm2
      
      BF=exp((BIC(lm1)-BIC(lm2))/2)
      return(BF)
   }

#------------------
#load data from file
shopData=read.csv(file="rawDataField.csv") #participants estimates
load("bonList.RData") #a list with receipts that contain the actual prices of each item and the respective category number

attach(shopData)   

#--------------
#some summary statistics

   AverageValue=sapply(bonList, mean)
   nItems=sapply(bonList,length)
   
   #(geometric mean of nItems)
   mNItems=exp(lm(log(nItems)~1)$coef)   #same as exp(mean(log(nItems))
   mean(nItems)
   median(nItems)
   summary(nItems)
   
   #(geometric) mean of true and estimated basket value
   mWarenwert=exp(lm(log(Warenwert)~1)$coef)
   mSchaetzung=exp(lm(log(Schaetzung)~1)$coef) 

   #(geometric) mean of a single item
   exp(mean(log(unlist(bonList))))
   median(unlist(bonList))
   mean(unlist(bonList))
   
   #mean underestimate
   1-mSchaetzung/mWarenwert #same as: 1-exp(mean(log(Schaetzung)-log(Warenwert)))

   #test underestimate for significance
   ome=log(Schaetzung/Warenwert) #same as (log(Schaetzung)-log(Warenwert))
   t.test(ome)
   mean(ome)

   #average subjective value of 1 CHF
   mSchaetzung/mWarenwert

   #overview of sample
   summary(as.factor(Datum))   
   summary(Altersrange)
   summary(Sex)
   nVpn=length(Schaetzung)



#Proportion of baskets that are underestimated
   t.test(Warenwert-Schaetzung)
   pUnderestimate=sum(Warenwert>Schaetzung)/nVpn
   binom.test(sum(Warenwert>Schaetzung), nVpn)
   #Bayes Factor via Savage-Dickey
   BF=1/dbinom(sum(Warenwert>Schaetzung), nVpn, .5)




#separate test for both sampling waves:

   #2011
   exp(mean(log(nItems[sample=="2011"])))
   exp(mean(log(Warenwert[sample=="2011"])))
   summary(Warenwert[sample=="2011"])
   t.test(Warenwert[sample=="2011"]-Schaetzung[sample=="2011"])#ome
   pUnderestimate=sum(Warenwert[sample=="2011"]>Schaetzung[sample=="2011"])/sum(sample=="2011")
   binom.test(sum(Warenwert[sample=="2011"]>Schaetzung[sample=="2011"]), sum(sample=="2011"))
   #BF via Savage-Dickey
   BF=1/dbinom(sum(Warenwert[sample=="2011"]>Schaetzung[sample=="2011"]), sum(sample=="2011"), .5)

   #2012
   exp(mean(log(nItems[sample=="2012"])))
   exp(mean(log(Warenwert[sample=="2012"])))
   summary(Warenwert[sample=="2012"])
   t.test(Warenwert[sample=="2012"]-Schaetzung[sample=="2012"])#ome
   pUnderestimate=sum(Warenwert[sample=="2012"]>Schaetzung[sample=="2012"])/sum(sample=="2012")
   binom.test(sum(Warenwert[sample=="2012"]>Schaetzung[sample=="2012"]), sum(sample=="2012"))
   #BF via Savage-Dickey
   BF=1/dbinom(sum(Warenwert[sample=="2012"]>Schaetzung[sample=="2012"]), sum(sample=="2012"), .5)



#-------------------
#Estimate quintiles

   qq=quantile(Warenwert, seq(0,1,.1))
   qqID=cut(Warenwert, breaks=qq, include.lowest=T)
   qqWarenwert=tapply(Warenwert,qqID, mean)
   qqEst=tapply(Schaetzung,qqID, mean)
   
   qqci95Warenwert=matrix(unlist(tapply(Warenwert,qqID, bootstrapCI)), nrow=2)
   qqci95Est=matrix(unlist(tapply(Schaetzung,qqID, bootstrapCI)), nrow=2)


#calculate bias
   bias=(Schaetzung-Warenwert)/Warenwert
   mean(bias)
   sd(bias)
   t.test(bias)
   median(bias)
   round(quantile(bias),2)


#--------
# Fit Steven's Power law

   steveLoss = function (params, true, perceived){
      #Loss function for steve's power law
      k = params[1] #constant
      alpha = params[2] #exponent
      
      pred=k*true^alpha
      
      loss=sum((perceived-pred)^2)
      return(loss)  
   }

   steveFit=nlminb(c(1,1), steveLoss, true=Warenwert, perceived=Schaetzung)
   exponent=steveFit$par[2]


#============
#Re-estiate stevens power law using log-log transformations
#http://mathbench.umd.edu/modules/misc_scaling/page11.htm
   logSchaetzung=log(Schaetzung) #estimate
   logWarenwert=log(Warenwert)   #true value
   
   loglog.Linear=lm(logSchaetzung~logWarenwert)
   exponent=loglog.Linear$coef[2]
   summary(loglog.Linear)

   #Interpretation
   beta=loglog.Linear$coef[2]
   exp(beta*log(1.1))  #10 % = 1.1
   #a 10% increase in the true value only yields a 9% increase in the estimated value
   #--> the higher the true value, the more unerestimation


#percent unterestimate for mean and median basket
   xx=c(mean=mean(Warenwert),quantile(Warenwert))
   pred=exp(loglog.Linear$coef[1]+loglog.Linear$coef[2]*log(xx))
   round(xx-pred,1) #absolute over/underestimate
   100*(xx-pred)/xx #percentage over/underestimate


#---------
#plot Data

   windows(5,5)
   par(mar=c(4.4,4.4,1,1))
   
   axisLim=c(0,200)
   
   #number of points not visible in the plot
   sum(Warenwert>200 | Schaetzung>200)
   
   plot(NULL, xlim=axisLim, ylim=axisLim, xlab=expression(paste("True basket value in Swiss Francs,  ",italic(y),"")), 
        ylab=expression(paste("Estimated basket value in Swiss Francs,   ",italic(hat(y)))))
   
   #add points
   points(Warenwert,Schaetzung, pch=1, cex=.8, col="grey", xlab=expression(paste("True sum,  ",italic(y),""))) #+runif(n=length(Schaetzung), min=-1, max=1)
   
   #add fitted loglog regression line
   abline(a=0,b=1, lty="dashed")
   yy=exp(predict(loglog.Linear))
   oo=order(Warenwert)
   points(Warenwert[oo],yy[oo], col="black", lwd=3, type="l")
   
   #add quartile points
   points(qqWarenwert,qqEst, cex=1.4, pch=21, bg="white",col="black" )
   arrows(qqWarenwert, qqci95Est[1,],qqWarenwert, qqci95Est[2,], angle=90, code=3, length=.02 )
   
   #add text
   atext=round(exp(loglog.Linear$coef[1]),2)
   btext=round(loglog.Linear$coef[2],2)
   equationtext=bquote(italic(hat(y))==.(atext)%*%italic(y)^.(btext))
   text(150,20, equationtext, cex=1.1)
   


#=====================
# Test influences on bias within a regression

   av=bias 

   #(optionally) only use data with complete observations (important to ensure that model comparison is based on same n)
   isMissing=is.na(age)|is.na(BarKarte) #21 missings
   isValid=!isMissing #or set isValid=TRUE
   
   #Basemodel
   baseModel=lm(av~1, subset=isValid)
   lm1=lm(av~logWarenwert, subset=isValid)
   getBF(baseModel, lm1)
  
   #Age
   lmAge=lm(av~logWarenwert+age, subset=isValid)
   summary(lmAge)
   getBF(lm1, lmAge) #older age yields more negative bias
   
   #interpret age coefficient
   coefs=summary(lmAge)$coefficients[,1]
   beta=coefs[3]
   10*beta

   
   #Number of items
   lmItems=lm(av~logWarenwert+age+nItems, subset=isValid)
   summary(lmItems)
   getBF(lmAge, lmItems) #underestimation larger if fewer items in the basket

   #Cash or Credit
   lmCash=lm(av~logWarenwert+age+nItems+BarKarte, subset=isValid)
   summary(lmCash)
   getBF(lmItems, lmCash) #no difference 

   #Higher moments: mean
   averagePrice=Warenwert/nItems
   lmAverage=lm(av~logWarenwert+averagePrice, subset=isValid)
   summary(lmAverage)
   getBF(lm1, lmAverage) #no effect

   #Higher moments: variance 
   va=sapply(bonList, var)
   lmVar=lm(av~logWarenwert+log(va), subset=isValid)
   summary(lmVar)
   getBF(lm1,lmVar)   #variance does not influence bias

   #Variance normalized by Warenwert (similar to coefficient of variation)
   cv=va/Warenwert
   lmVar2=lm(av~logWarenwert+log(cv), subset=isValid)
   summary(lmVar2)
   getBF(lm1,lmVar2)   #variance does not influence bias

   #Higher moments: skewness 
   skew=sapply(bonList, skewness)
   logSkew=lm(av~logWarenwert+skew, subset=isValid)
   getBF(lm1,logSkew) #skewness does not influence bias    

   #Max item
   maxItem=sapply(bonList, max)
   logMax=lm(av~logWarenwert+log(maxItem), subset=isValid)
   getBF(lm1,logMax) #maximum item does not influence bias    

   #overview over the most important models
   lm1=lm(av~logWarenwert, subset=isValid)
   lm2=lm(av~logWarenwert+age, subset=isValid)
   lm3=lm(av~logWarenwert+age+nItems, subset=isValid)
   lm4=lm(av~logWarenwert+age+nItems+BarKarte, subset=isValid)
   anova(lm1,lm2,lm3,lm4)



#=======
# Analyze estimate for the number of items (only 2012 data)

   #Is the number of items underestimated? --> No! Rather overestimation
   mean(nItems-Nguess, na.rm=T)
   median(nItems-Nguess, na.rm=T)
   sd(nItems-Nguess, na.rm=T)

   t.test(nItems-Nguess) #average overestimation = 1.9 items
   n2012=sum(!is.na(Nguess))
   binom.test(sum(nItems<Nguess, na.rm=T), n2012)



   #add n estimate into the regression
   is2012=sample=="2012" 
   use2012=is2012&isValid


   lm1.2012=lm(av~logWarenwert, subset=use2012)
   lm3.2012=lm(av~logWarenwert+age+nItems, subset=use2012)
   lm8.2012=lm(av~logWarenwert+age+nItems+Nguess, subset=use2012)
   summary(lm8.2012)   
   getBF(lm3.2012, lm8.2012) 

   #include Reihenfolge (# first vs $ first)
   lm9.2012=lm(av~logWarenwert+age+nItems++Nguess+Reihenfolge, subset=use2012)
   getBF(lm8.2012,lm9.2012)   # BF < 1

   #asking n first or last
   t.test(av~Reihenfolge)

   #include Shoppinglist
   sum(Einkaufsliste, na.rm=T)/n2012
   lm10.2012=lm(av~logWarenwert+age+nItems+Nguess+Einkaufsliste, subset=use2012)
   getBF(lm8.2012,lm10.2012)   # BF < 1


#------------------------
# Estimate effect of price endings


   #Distribution of price endings
   allItems=unlist(bonList)
   rappen=round(allItems%%1, 2) # round to get rid of prices with more than 2 digits due to fresh vegis that are priced by the gram
   uniquePrices=sort(unique(rappen))
   histData=hist(rappen,breaks=c(uniquePrices-.001,1), plot=F) 
   rappenTable=cbind(category=uniquePrices,n=histData$counts)

   #Plot distribution
   windows()
   plot(rappenTable[,"category"], rappenTable[,"n"], type="h", lwd=1, bty="l" ,
        main="Distribution of Price Endings", xlab="Price ending", ylab="Frequency")
   points(rappenTable[,"category"], rappenTable[,"n"], col="darkblue", pch=16, cex=1)


   #number of endings >.5 
   countEndings = function (vec, crit=.5) {
      #counts how many endings are >= crit
      rap=round(vec%%1, 2)
      isBigger=sum(rap>=crit)
      return(isBigger)
   }
   
   rappenEndings=sapply(bonList, countEndings)
   pRappen=rappenEndings/nItems

   lm.rappen=lm(av~logWarenwert+age+nItems+pRappen, subset=isValid)
   getBF(lm3,lm.rappen)



#======================
# Analysis of product categories  
# (only for 2012 data because in 2011 the product categories assigned by the retailer were slightly different)
   
   #load list with product categories from file 
   load("productCategory.RData")

   
   #idx of alcohol and sweets (from retailer's database)
   alkIdx=c(3801, 3802, 3902, 3903, 3904) #rather expensive
   sweetIdx=c(3501, 3502, 3503, 3504)
   

   #idenfity alcohol and sweets
   nAlk=rep(NA, times=nVpn)
   nSweets=rep(NA, times=nVpn)
   
   for (bb in 1:nVpn) {
      thisIdx=idxBonList[[bb]]
      nAlk[bb]=length(intersect(thisIdx,alkIdx))
      nSweets[bb]=length(intersect(thisIdx,sweetIdx))
   }


   #proportion of sweets and alcohol in each basket
   pAlk=nAlk/nItems
   pSweets=nSweets/nItems

   #average number of sweets or alk 
   meanAlk=mean(nAlk[is2012])
   meanSweets=mean(nSweets[is2012])

   #overview
   sum(nAlk[is2012]>0)/sum(is2012) #30%
   sum(nSweets[is2012]>0)/sum(is2012) #40%

   #simple factor for lm
   hasAlk=factor(nAlk>0)
   hasSweets=factor(nSweets>0)

   #covariates (more items = higher chance for sweets and alcohol)
   cor.test(as.numeric(nAlk>0)[is2012],nItems[is2012]) 
   cor.test(as.numeric(nSweets>0)[is2012],nItems[is2012]) 


   #test influence on av
   lmX.2012=lm(av~logWarenwert+age+nItems+Nguess, subset=use2012)
   lmAlk.2012=lm(av~logWarenwert+age+nItems+Nguess+hasAlk, subset=use2012)
   getBF(lmX.2012,lmAlk.2012)

   lmSweets.2012=lm(av~logWarenwert+age+nItems+Nguess+hasSweets, subset=use2012)
   getBF(lmX.2012,lmSweets.2012)

   #--> Alk and Sweets have no effect above and beyond the other predictors



#--------------------------------------------------------
#Primacy and recency effects based on order on the coupon 

   #include first and last item scanned as predictor
   #first scanned = first out = last purchased
   #items scanned last are purchased first 
   
   lastPurchased=sapply(bonList,function(x) x[1])
   firstPurchased=sapply(bonList,function(x) x[length(x)])
   
   #no difference in price
   t.test(lastPurchased-firstPurchased)
   
   #apply to whole sample
   lm3   =lm(av~logWarenwert+age+nItems              , subset=isValid)
   lm.end=lm(av~logWarenwert+age+nItems+lastPurchased, subset=isValid)
   summary(lm.end)
   getBF(lm3,lm.end) #small effect
   
   lm.first=lm(av~logWarenwert+age+nItems+firstPurchased, subset=isValid)
   getBF(lm3,lm.first) #no effect


   
   
 

#------------------------------
#Overview of BIC, BF, and LogLik for all relevant models 

   isMissing=is.na(age)|is.na(BarKarte) #21 missings
   isValid=!isMissing 
   
   base=lm(av~1, subset=isValid)
   lm1=lm(av~logWarenwert, subset=isValid)
   lm2=lm(av~logWarenwert+age, subset=isValid)
   lm3=lm(av~logWarenwert+age+nItems, subset=isValid)
   
   modelList=list(base,lm1,lm2,lm3)
   
   #BIC and LogLik
   BIC(base,lm1,lm2,lm3)
   ll=sapply(modelList, logLik)
   round(ll)
   
   #draw BF Table
   bfTable=matrix(NA, nrow=3, ncol=4)
   for(i in 1:3){
      for(j in (i+1):4){
         bfTable[i,j]=getBF(modelList[[i]], modelList[[j]])   
      }
   }

   round(bfTable)
   