#analyze lab data using Stan
library(coda)
library(rstan)
library(loo) 
source("plotWeighting.r") #funtion to plot the quadratic weighting vector
source("plotPostPred.r")  #funtion to plot the posterior predictive

#function to calculate bootstrap
bootstrapCI = function (vec, conf=.95, nSamples=30000 ) {
  # calculate bootstrapped CI of vec 
  # nSamples = number of bootstrap samples to be drawn
  # conf= 95% confidence interval (2-sided)
  
  twoSided=(1-conf)/2
  bootMean=sapply(1:nSamples, function(i) { mean(sample(vec, replace = TRUE))})
  ci=quantile(bootMean,probs=c(twoSided,1-twoSided)) 
  return(ci=ci)  
}


#Stan parameters
iter=108000
chains=4
warmup=8000
thin=50

rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())


# Prepare data for STAN in long format ---------------

  load("labData.RData")
  n=length(rawDatalist)

   itemMatrix=NULL
   guessVec=NULL
   nValid=std=rep(NA, times=n)

   for (i in 1:n) {
      
      this.rawData   =rawDatalist[[i]]
      this.itemMatrix=data.matrix(this.rawData[,14:37]) #each row = one of 75 sequences
      this.Guess     =this.rawData$geschtzter.Betrag  
      
      #sort itemMatrix and guesses based on the order in which they were presented
      sortIdx        =order(this.rawData$originalOrder)
      this.itemMatrix=this.itemMatrix[sortIdx,]
      this.Guess     =this.Guess[sortIdx]

      #identify and remove outliers (bias > 1)
      this.actual =rowSums(this.itemMatrix)      
      this.bias   =(this.Guess-this.actual)/this.actual
      this.outlier=this.bias>1
      
      nOutliers=sum(this.outlier)
      nValid[i]=dim(this.itemMatrix)[1]-nOutliers
      
      if(nOutliers>0) {
         outlierPos     =which(this.outlier==T)
         this.Guess     =this.Guess[-outlierPos] 
         this.itemMatrix=this.itemMatrix[-outlierPos,]
      }
      
      std[i]=sd(log(this.Guess))
      
      #append data 
      itemMatrix=rbind(itemMatrix, this.itemMatrix)
      guessVec  =c(guessVec, this.Guess)
   }  

   #set index for the long format   
   upperIdx=cumsum(nValid)
   lowerIdx=(upperIdx-nValid)+1

   
#-----------------------------------------------
#Estimate STAN models 
   
   
# No weighting - Power first -------------------
 
   #datalist
   stanData =list( 
      N           = n, #number of participants
      nTrials     = length(guessVec),
      upperIdx    = upperIdx,
      lowerIdx    = lowerIdx,
      itemMatrix  = itemMatrix,
      guessVec    = guessVec
   )
   
   initFunc <-function (i) {
      initList=list()
      for (ll in 1:i){
         initList[[ll]] = list (
            b1_mu          = 1,
            exponent_mu    = 1,

            b1             = rep(1,times=n),
            exponent       = rep(1, times=n),

            sigma_b1       = .01,
            sigma_exponent = .01,

            std            = rep(1, times=n)
         )
      }
      
      return(initList)
   }
   

   fit.Pfirst.NoWeight <- stan(file   = 'noWeightingPowerFirst.stan', 
                               data   = stanData, 
                               iter   = iter,  
                               chains = chains, 
                               warmup = warmup, 
                               init   = initFunc(chains), 
                               thin   = thin) 
                  
   pars=c("b1_mu", "exponent_mu", "sigma_b1", "sigma_exponent")
   print(fit.Pfirst.NoWeight, pars=pars)  
   plot(fit.Pfirst.NoWeight,pars=pars)
   #pairs(fit.Pfirst.NoWeight, c("b1[2]", "exponent[2]"))
   traceplot(fit.Pfirst.NoWeight,pars=c("b1[1]", "exponent[1]","b1_mu", "sigma_b1", "exponent_mu", "sigma_exponent", "sigma_b1") )
   
   #Residual plot: Model prediction against residual error 
   post=extract(fit.Pfirst.NoWeight, pars=c("pred", "std"))
   
   medianPred =apply(post$pred,2,median)
   logResidual=log(medianPred)-log(stanData$guessVec)
   logPred    =log(medianPred)
   xrange     =range(logPred)
   yrange     =range(logResidual)
   
   windows()
   plot(logPred, logResidual, xaxt="n", xlim=xrange, ylim=yrange, xlab="log(model predictions)",
        ylab="log(Model predictions) - log(observed values)", main="Residual Plot (log scale)")
   
   regObj=lm(logResidual~logPred)
   abline(regObj, col="red", lwd=2)
   
   #add axis
   axLabels=seq(0,500, 50)
   axPos=log(axLabels)
   axis(1, at=axPos, labels = axLabels)
   axis(2, at=axPos, labels = axLabels)
   
   savePlot("residualPlot.Pfirst.NoWeight.eps", type="eps")
   savePlot("residualPlot.Pfirst.NoWeight.pdf", type="pdf")
   
   
   
# No weighting - Sum first ------------------

   fit.Sfirst.NoWeight <- stan(file   = 'noWeightingSumFirst.stan', 
                               data   = stanData, 
                               iter   = iter,  
                               chains = chains, 
                               warmup = warmup, 
                               init   = initFunc(chains), 
                               thin   = thin) 
   
   
   pars=c("b1_mu", "exponent_mu", "sigma_b1", "sigma_exponent")
   print(fit.Sfirst.NoWeight,pars=pars)
   
   plot(fit.Sfirst.NoWeight,pars=pars)
   traceplot(fit.Sfirst.NoWeight,pars=c("b1[1]", "exponent[1]","b1_mu", "sigma_b1", "exponent_mu", "sigma_exponent", "sigma_b1") )
   
   plotPostPred(observed=stanData$guessVec,stanFit=fit.Sfirst.NoWeight,filename="sumFirst.noWeight.pdf")
   

# Linear weighting - Sum first ------------------
   
   #dummy matrix for linear weighting
   dummyVec=((1:24)-mean(1:24))/sd(1:24)

   
   #datalist
   stanData =list( 
      N           = n, #number of participants
      nTrials     = length(guessVec),
      dummyVec    = dummyVec,
      upperIdx    = upperIdx,
      lowerIdx    = lowerIdx,
      itemMatrix  = itemMatrix,
      guessVec    = guessVec
   )
   
   initFunc <-function (i) {
      initList=list()
      for (ll in 1:i){
         initList[[ll]]= list (
            b1_mu          = 1,
            exponent_mu    = 1,
            w1_mu          = 0,

            b1             = rep(1,times=n),
            exponent       = rep(1, times=n),
            w1             = rep(0, times=n),

            sigma_b1       = .01,
            sigma_exponent = .01,
            sigma_w1       = .01,

            std            = rep(1, times=n)
         )
      }
      
      return(initList)
   }
   
   fit.Sfirst.Linear <- stan(file    = 'linearWeightingSumFirst.stan', 
                             data    = stanData, 
                             iter    = iter,
                             chains  = chains, 
                             warmup  = warmup, 
                             init    = initFunc(chains), 
                             thin    = thin) 
                  

   pars=c("b1_mu", "exponent_mu", "w1_mu","sigma_b1", "sigma_exponent",  "sigma_w1")
   print(fit.Sfirst.Linear,pars=pars)
   
   plot(fit.Sfirst.Linear,pars=c("b1", "exponent", "std"))
   pairs(fit.Sfirst.Linear, pars = c("b1[2]", "exponent[2]"))
   pairs(fit.Sfirst.Linear, pars = c("b1[5]", "w1[5]"))
   traceplot(fit.Sfirst.Linear,pars=c("b1[1]", "exponent[1]","b1_mu", "sigma_b1", "exponent_mu", "sigma_exponent", "sigma_b1", "w1_mu", "sigma_w1") )
   
 
# Linear Power First ------------------

   fit.Pfirst.Linear <- stan(file   = 'linearWeightingPowerFirst.stan',
                             data   = stanData, 
                             iter   = iter,
                             chains = chains, 
                             warmup = warmup, 
                             init   = initFunc(chains), 
                             thin   = thin)
         
   print(fit.Pfirst.Linear,pars=pars)
   
   plot(fit.Pfirst.Linear,pars=c("b1", "exponent", "std"))
   pairs(fit.Pfirst.Linear, pars = c("b1[2]", "exponent[2]"))
   pairs(fit.Pfirst.Linear, pars = c("b1[5]", "w1[5]"))
   traceplot(fit.Pfirst.Linear,pars=c("b1[1]", "exponent[1]","b1_mu", "sigma_b1", "exponent_mu", "sigma_exponent", "sigma_b1", "w1_mu", "sigma_w1") )
   
   plotPostPred(observed=stanData$guessVec,stanFit=fit.Pfirst.Linear,filename="powerFirst.linearWeight.pdf")
   
   #-----------------------------------------------------------------
   #Generate model predictions for each sequence and each participant
   
   post=extract(fit.Pfirst.Linear, pars=c("b1", "exponent", "w1"))
   
   #individual-level parameters - use median to make predictions
   b1       = apply(post$b1,2,median)
   exponent = apply(post$exponent,2,median)
   w1       = apply(post$w1,2,median)
   
   #generate individual-level predictions
   modelpredLinear=rep(NA, times=length(b1)*75)
   
   for (i in 1:length(b1))  { #loop across all participants
     
     this.rawData   = rawDatalist[[i]]
     this.itemMatrix= data.matrix(this.rawData[,14:37]) #each row = one of 75 sequences
     this.Guess     = this.rawData$geschtzter.Betrag  
     this.actual    = rowSums(this.itemMatrix)
     
     #construct individual weight vector
     weightVec = b1[i]+w1[i]*dummyVec
     
     #loop through all sequences
     idx=(1:75)+75*(i-1)
     for (s in 1:75)  modelpredLinear[idx[s]] = sum(weightVec*(this.itemMatrix[s,]^exponent[i]))
   }
   
   #add predictions to longFrame
   longFrame$predLinear=modelpredLinear
   

# Quadratic weighting - Power first ------------------

   #datalist
   stanData =list( 
      N            = n, #number of participants
      nTrials      = length(guessVec),
      dummyVec     = dummyVec,
      dummySquared = dummyVec^2,
      upperIdx     = upperIdx,
      lowerIdx     = lowerIdx,
      itemMatrix   = itemMatrix,
      guessVec     = guessVec
   )
   
   initFunc <-function (i) {
      initList=list()
      for (ll in 1:i){
         initList[[ll]]= list (
            b1_mu       = 1,
            exponent_mu = 1,
            w1_mu       = 0,
            w2_mu       = 0,
            
            b1          = rep(1,times=n),
            exponent    = rep(1, times=n),
            w1          = rep(0, times=n),
            w2          = rep(0, times=n),
            
            sigma_b1       = .1,
            sigma_exponent = .1,
            sigma_w1       = .1,
            sigma_w2       = .1,
            
            std            = rep(1, times=n)
         )
      }
      
      return(initList)
   }
   
   fit.Pfirst.Quadratic <- stan(file   = 'quadraticWeightingPowerFirst.stan', data = stanData, 
                                iter   = iter, 
                                chains = chains, 
                                warmup = warmup, 
                                init   = initFunc(chains), 
                                thin   = thin)


   pars=c("b1_mu", "exponent_mu", "w1_mu","w2_mu","sigma_b1", "sigma_exponent",  "sigma_w1", "sigma_w2")
   print(fit.Pfirst.Quadratic,pars=pars)
   
   plot(fit.Pfirst.Quadratic,pars=c("b1", "exponent", "std"))
   pairs(fit.Pfirst.Quadratic, pars = c("b1[2]", "exponent[2]"))
   pairs(fit.Pfirst.Quadratic, pars = c("b1[5]", "w1[5]"))
   traceplot(fit.Pfirst.Quadratic,pars=c("b1[1]", "exponent[1]","b1_mu", "sigma_b1", "exponent_mu", "sigma_exponent", "sigma_b1", "w1_mu", "sigma_w1","w2_mu", "sigma_w2") )
   
   #plot posterior predictive on individual level
   plotPostPred(observed=stanData$guessVec,stanFit=fit.Pfirst.Quadratic,filename="powerFirst.quadraticWeight.pdf")
   

   #plot weighting vector
   plotWeighting(fit.Pfirst.Quadratic, dummyVec)
   savePlot(file="Figure3.pdf", type="pdf")
   savePlot(file="Figure3.eps", type="eps")
   
   
 
   #-----------------------------------------------------------------
   #Generate model predictions for each sequence and each participant
   
   post=extract(fit.Pfirst.Quadratic, pars = c("b1_mu", 
                                               "w1_mu", 
                                               "w2_mu", 
                                               "exponent_mu",
                                               "b1",
                                               "w1",
                                               "w2",
                                               "exponent"))
   
   #individual-level parameters - use median to make predictions
   b1       = apply(post$b1,2,median, na.rm=T)
   exponent = apply(post$exponent,2,median, na.rm=T)
   w1       = apply(post$w1,2,median, na.rm=T)
   w2       = apply(post$w2,2,median, na.rm=T)
   
   #generate the weighting vector
   modelpredQuadratic=rep(NA, times=length(longFrame$guess))
   for (i in 1:length(b1))  { #loop across all participants
     
     this.rawData    = rawDatalist[[i]]
     this.itemMatrix = data.matrix(this.rawData[,14:37]) #each row = one of 75 sequences
     this.Guess      = this.rawData$geschtzter.Betrag  
     this.actual     = rowSums(this.itemMatrix)

     #construct individual weight vector
     weightVec = b1[i]+w1[i]*dummyVec+w2[i]*dummyVec^2
     
     #loop through all sequences
     idx=(1:75)+75*(i-1)
     for (s in 1:75)  modelpredQuadratic[idx[s]] = sum(weightVec*(this.itemMatrix[s,]^exponent[i]))
   }
   
   #add predictions to longFrame
   longFrame$predQuadratic=modelpredQuadratic
   
   #alternative approach: use the median of the model predictions from stan directly (almost exactly the same result)
    #post=extract(fit.Pfirst.Quadratic)
    #pred=apply(post$pred,2,median)
   

   #-----------------------------------------------------
   #Combine the model predictions with the longFrame data
 
   #calculate bias as Order of Signed Magnitude Error (OSMD)
   longFrame$logBias            = log(longFrame$guess/longFrame$actual)
   longFrame$modelbias.linear   = log(longFrame$predLinear/longFrame$actual)
   longFrame$modelbias.quadratic= log(longFrame$predQuadratic/longFrame$actual)
   
   
   #remove outliers (to match the modeled data)
   longFrame$bias = (longFrame$guess-longFrame$actual)/longFrame$actual 
   longFrame = longFrame[longFrame$bias<1,]
  
   #-----------------
   #Plot the observed bias against the model-predicted bias depending on frequency distribution and number sequence 
   
   returnMeanTable = function (bias){
     #function that returns the mean and ci95 for the bias depending on the experimental condition

     #mean for each sequence ("verlauf")
     meanSeq = tapply(bias, list(longFrame$verlauf), mean)
     
     #bootstrap across individual means
     biasMatrix1= tapply(bias,list(longFrame$verlauf, longFrame$vpn), mean )
     ciSeq      = apply(biasMatrix1, 1, bootstrapCI)
     
     #mean for each distribution ("verteilung")
     meanDist = tapply(bias, list(longFrame$verteilung), mean)

     #bootstrap across individual means
     biasMatrix2 = tapply(bias,list(longFrame$verteilung, longFrame$vpn), mean )
     ciDist      = apply(biasMatrix2, 1, bootstrapCI)
     
     #glue it together
     big   = c(meanDist[6], NA, meanDist[1:5], NA,NA, meanSeq[1:4], NA)
     bigCI = cbind(ciDist[,6], NA, ciDist[,1:5], NA,NA, ciSeq[,1:4], NA)
     
     names(big)[3] ="Uniform"
     names(big)[4] ="Positive skew"
     names(big)[5] ="Negative skew"
     names(big)[6] ="Unimodal"
     names(big)[7] ="Bimodal"
     names(big)[10]="Increasing"
     names(big)[11]="Decreasing"
     names(big)[12]="Peaked"
     names(big)[13]="U-shaped"
     
     return(list(big=big, bigCI=bigCI))
   }
   
   observedBias        = returnMeanTable(longFrame$logBias)
   modelBias.quadratic = returnMeanTable(longFrame$modelbias.quadratic)
   modelBias.linear    = returnMeanTable(longFrame$modelbias.linear)
   
   bigMean=rbind(pred.linear    = modelBias.linear$big,
                 pred.quadratic = modelBias.quadratic$big,
                 observed       = observedBias$big)
   
  
   
   #draw the barplot
   windows(6,6)
   par(mar=c(5,2,4,7))
   xlim=range(0,modelBias.quadratic$bigCI,observedBias$bigCI,bigMean, na.rm=T)
   ypos=barplot(bigMean,xlim=xlim, horiz=T, ylab="", xaxt="n", beside=T, col.axis="white", space=c(0,1), col=c("firebrick1", "gold", "forestgreen"))
   legend(x=mean(xlim), y=max(ypos)+11, xpd=T,fill=c("forestgreen", "gold", "firebrick1"),  legend=c("observed data", "model prediction (quadratic weighting)","model prediction (linear weighting)"), xjust=.5)
   
   arrows(observedBias$bigCI[1,]       , ypos[3,],observedBias$bigCI[2,]       ,ypos[3,], code=3, length=.02, angle=90, lwd=1.2)
   arrows(modelBias.quadratic$bigCI[1,], ypos[2,],modelBias.quadratic$bigCI[2,],ypos[2,], code=3, length=.02, angle=90,lwd=1.2)
   arrows(modelBias.linear$bigCI[1,]   , ypos[1,],modelBias.linear$bigCI[2,]   ,ypos[1,], code=3, length=.02, angle=90,lwd=1.2)
   
   axis(1, at=seq(round(max(c(0,xlim[2]))),round(xlim[1],2), -0.02))
   mtext(expression(paste("Estimation bias:  ",log(hat(italic(y))/italic(y)))), side=1, line=3.2)
   abline(v=0)
   
   #add labels
   barLab=colnames(bigMean)
   barLab[1]=expression(bold(Baseline))
   
   xpos=max(xlim)+0.003
   text(x=xpos, y=colMeans(ypos) , labels=barLab, adj=0, xpd=T)
   text(x=xpos, y=mean(ypos[,14]), labels=expression(bold(Sequence)), adj=0, xpd=T)
   text(x=xpos, y=mean(ypos[,8]) , labels=expression(bold(Distribution)), adj=0, xpd=T)
   
 
   savePlot(file="Figure2.posteriorPredictive.pdf", type="pdf")
   savePlot(file="Figure2.posteriorPredictive.eps", type="eps")
   
  
    
# Quadratic Weighting - Sum First -------------
   
   fit.Sfirst.Quadratic <- stan(file   = 'quadraticWeightingSumFirst.stan', 
                                data   = stanData, 
                                iter   = iter, 
                                chains = chains, 
                                warmup = warmup, 
                                init   = initFunc(chains), 
                                thin   = thin)
   
   print(fit.Sfirst.Quadratic,pars=c("b1_mu", "exponent_mu", "w1_mu",  "w2_mu","sigma_b1", "sigma_exponent",  "sigma_w1", "sigma_w2"))
   
   plot(fit.Sfirst.Quadratic,pars     = c("b1", "exponent", "std"))
   pairs(fit.Sfirst.Quadratic, pars   = c("b1[2]", "exponent[2]"))
   pairs(fit.Sfirst.Quadratic, pars   = c("b1[5]", "w1[5]"))
   traceplot(fit.Sfirst.Quadratic,pars= c("b1[1]", "exponent[1]","b1_mu", "sigma_b1", "exponent_mu", "sigma_exponent", "sigma_b1", "w1_mu", "sigma_w1","w2_mu", "sigma_w2") )
   
   #plotPostPred(observed=stanData$guessVec,stanFit=fit.Sfirst.Quadratic,filename="sumFirst.quadraticWeight.pdf")
   
   #plot weighting vector
   #plotWeighting(fit.Sfirst.Quadratic, dummyVec)
   
   
#=============
# Compare Models --------------

   returnLoo  = function (fit) loo(extract_log_lik(fit)) 
   returnWAIC = function (fit) waic(extract_log_lik(fit)) 
   
   #===== LOO

   #Sum First 
   loo.Sfirst.NoWeight = returnLoo(fit.Sfirst.NoWeight)
   loo.Sfirst.Linear   = returnLoo(fit.Sfirst.Linear)
   loo.Sfirst.Quadratic= returnLoo(fit.Sfirst.Quadratic)
  
   # Power first
   loo.Pfirst.NoWeight  = returnLoo(fit.Pfirst.NoWeight)
   loo.Pfirst.Linear    = returnLoo(fit.Pfirst.Linear)
   loo.Pfirst.Quadratic = returnLoo(fit.Pfirst.Quadratic)
  
   
   compare(loo.Sfirst.NoWeight, 
           loo.Sfirst.Linear, 
           loo.Sfirst.Quadratic, 
           
           loo.Pfirst.NoWeight, 
           loo.Pfirst.Linear,
           loo.Pfirst.Quadratic)
   
   
   #===== WAIC
   
   #Sum First 
   waic.Sfirst.NoWeight = returnWAIC(fit.Sfirst.NoWeight)
   waic.Sfirst.Linear   = returnWAIC(fit.Sfirst.Linear)
   waic.Sfirst.Quadratic= returnWAIC(fit.Sfirst.Quadratic)
  
   # Power first
   waic.Pfirst.NoWeight  = returnWAIC(fit.Pfirst.NoWeight)
   waic.Pfirst.Linear    = returnWAIC(fit.Pfirst.Linear)
   waic.Pfirst.Quadratic = returnWAIC(fit.Pfirst.Quadratic)
   
   compare(waic.Sfirst.NoWeight, 
           waic.Sfirst.Linear, 
           waic.Sfirst.Quadratic,
           
           waic.Pfirst.NoWeight,
           waic.Pfirst.Linear,
           waic.Pfirst.Quadratic)
   
   
   #compare log-likelihoods
   
   returnLL = function(fit){
     
     ll=extract(fit, "log_lik")$log_lik
     meanLL=mean(ll)
     maxLL=max(rowSums(ll)) #chain that generated the highest (log)likelihood
     maxLLidx=which(maxLL==rowSums(ll))[1] 
     return(c(mean=meanLL, max=maxLL, idx=maxLLidx))
   }
   
   ll=rbind( ll.Pfirst.quadratic = returnLL(fit.Pfirst.Quadratic),
             ll.Sfirst.quadratic = returnLL(fit.Sfirst.Quadratic),
             ll.Pfirst.linear    = returnLL(fit.Pfirst.Linear),
             ll.Sfirst.linear    = returnLL(fit.Sfirst.Linear),
             ll.Sfirst.noWeight  = returnLL(fit.Sfirst.NoWeight),
             ll.Pfirst.noWeight  = returnLL(fit.Pfirst.NoWeight)
           )
   
   #display log-likelihood in decreasing order (best model on top)
   ll[order(ll[,"mean"],decreasing=T ),]
 
#-----check if all models yield an exponent < 1---- 
   testExponent = function (fit, varName="exponent_mu", compVal=1){
      library(coda)
      library(MASS)
      
      ee=unlist(extract(fit, varName))
      hpd=HPDinterval(as.mcmc(ee))
      mm=mean(ee)
      
      #savage dickey density ratio
      fitPar=fitdistr(ee, "normal")
      
      #plot posterior
      hist(ee, breaks=40, col="grey90", prob=T, xlim=c(range(c(compVal,ee))), main="")
      abline(v=compVal, lty="dashed")
      xx=seq(min(ee),max(ee),length.out=1000)
      yy=dnorm(xx,mean=fitPar$estimate["mean"], sd=fitPar$estimate["sd"])
      points(xx,yy, type="l", col="green", lwd=3)
      
      p=sum(ee<1)/length(ee)
         
      #add prior distribution to plot
      points(xx,dnorm(xx, mean=1, sd=5), type="l", col="red", lwd=3)
      densUnderPrior=dnorm(1, mean=compVal, sd=5) #prior copy-pasted from STAN model file
      
      BFzero=densUnderPrior/dnorm(1,mean=fitPar$estimate["mean"], sd=fitPar$estimate["sd"])
      mtext(paste("p(param<1) =", signif(p,3),";   BF =",signif(BFzero,2) ))
      return(c(mean=mm,low95=hpd[1], high95=hpd[2], p=p, BF=BFzero))
   }
   
  
   windows(10,6)
   layout(matrix(1:8, nrow=2, byrow = T))
   testExponent(fit.Sfirst.NoWeight) ; mtext("Sfirst.NoWeight" , line=2)
   testExponent(fit.Sfirst.Linear)   ; mtext("Sfirst.Linear"   , line=2)
   testExponent(fit.Sfirst.Quadratic); mtext("Sfirst.Quadratic", line=2)
  
   testExponent(fit.Pfirst.NoWeight) ; mtext("Pfirst.NoWeight" , line=2)
   testExponent(fit.Pfirst.Linear)   ; mtext("Pfirst.Linear"   , line=2)
   testExponent(fit.Pfirst.Quadratic); mtext("Pfirst.Quadratic", line=2)
  
   #same for intercept
   windows(10,6)
   layout(matrix(1:8, nrow=2, byrow = T))
   testExponent(fit.Sfirst.NoWeight, varName="b1_mu") ; mtext("Sfirst.NoWeight", line=2)
   testExponent(fit.Sfirst.Linear, varName="b1_mu")   ; mtext("Sfirst.Linear", line=2)
   testExponent(fit.Sfirst.Quadratic, varName="b1_mu"); mtext("Sfirst.Quadratic", line=2)
   
   testExponent(fit.Pfirst.NoWeight, varName="b1_mu") ; mtext("Pfirst.NoWeight", line=2)
   testExponent(fit.Pfirst.Linear, varName="b1_mu")   ; mtext("Pfirst.Linear", line=2)
   testExponent(fit.Pfirst.Quadratic, varName="b1_mu"); mtext("Pfirst.Quadratic", line=2)

