#Analyze data from labstudy
library(lme4)

#function to calculate bootstrap
bootstrapCI = function (vec, conf=.95, nSamples=30000 ) {
  # calculate bootstrapped CI of vec 
  # nSamples = number of bootstrap samples to be drawn
  # conf= 95% confidence interval (2-sided)
  
  twoSided=(1-conf)/2
  bootMean=sapply(1:nSamples, function(i) { mean(sample(vec, replace = TRUE))})
  ci=quantile(bootMean,probs=c(twoSided,1-twoSided)) 
  return(ci=ci)  
}

load("labData.RData")

 
#define dependent variables
longFrame$bias =   (longFrame$guess-longFrame$actual)/longFrame$actual #log(longFrame$guess/longFrame$actual)
   
#Treat cases where bias>1 as outliers (16 obervations) - bias > 1 indicates more than 100% overestimate
   removePos=longFrame$bias>1
   longFrame=longFrame[!removePos,]


#Plot bias
hist(longFrame$bias, breaks=50, col="lightblue", xlim=c(-1,1), prob=T)
abline(v=0, lty="dashed", col="red")

#superimpose normal distribution
m=mean(longFrame$bias)
sd=sd(longFrame$bias)
xx=seq(-1,1, .01)
lines(xx,dnorm(xx,m,sd), lwd=2 )

meanBias=tapply(longFrame$bias,longFrame$vpn, mean)
hist(meanBias, breaks=20, col="lightgrey", prob=T)
abline(v=0, lty="dashed", col="red")

#superimpose normal distribution
xx=seq(-1,1, .01)
lines(xx,dnorm(xx,mean(meanBias),sd(meanBias)), lwd=2 )

t.test(meanBias)
sd(meanBias)

#more conservative binomial test
binom.test(x=sum(meanBias>0), n=40)


#-----------------
# Analyze predictors in more detail

attach(longFrame)


#Proportion of baskets that are underestimated
mean(bias<0)


#relationship between bias and accuracy in the letter task

  hist(vpnData$nCorrectLetter)
  #across individual participants
  mBias=tapply(bias, vpn, mean)
  
  boxplot(vpnData$nCorrectLetter)
  plot(vpnData$nCorrectLetter,mBias)
  summary(lm(mBias~vpnData$nCorrectLetter))
  
  #within each participant (across trials)
  lmer(bias~letterCorrect+(letterCorrect|vpn), REML=F)
  e=abs(bias)
  lmer(e~letterCorrect+(letterCorrect|vpn), REML=F)
  
  #--> accuracy in the letter task does not predict bias within nor between (does not predict error - abs(bias) - either)
  
#--------
#fit regression within subjects 


#test if exponential or linear provides a better fit to the data
   logGuess=log(guess)
   logActual=log(actual)
   
   #null-model
   logBase=lmer(logGuess~1+(logActual|vpn), REML=F)
   
   #log model
   logLm=lmer(logGuess~logActual+(logActual|vpn), REML=F)
   summary(logLm)
   
   
   #model comparison based on BIC and BF
   BIC(logBase)-BIC(logLm) #--> positive difference = model on the right is better
   anova(logBase,logLm)
   BF=exp((BIC(logBase)-BIC(logLm))/2) #logLm much more likely than logBase

#interpret coefficients
   coefs=summary(logLm)$coefficients[,1]
   beta =coefs[2]
   exp(beta*log(1.1))  #10 % = 1.1
   #a 10% increase in the true value only yields a 9.6% increase in the estimated value
   #--> the higher the true value, the more unerestimation


#--------
# PLOT Data
   windows(5,5)
   par(mar=c(4.4,4.4,1,1))
   axLim=range(c(actual,guess ))
   plot(actual,guess, xlim=c(0,500), ylim=c(0,700), pch=1, cex=.8, col="grey", xlab=expression(paste("True sum,  ",italic(y),"")), ylab=expression(paste("Estimated sum,  ", italic(hat(y)))))
   abline(0,1, lty="dashed")
   
   yy=exp(coefs[1])*actual^coefs[2] #same as: yy=exp(coefs[1]+logActual*coefs[2])
   
   oo=order(actual)
   points(actual[oo], yy[oo], type="l", col="black", lwd=2)

   # Calculate quintiles for each individual
   qInd.list=tapply(actual, vpn, quantile, seq(0,1,.2))
   qInd=sapply(qInd.list, function(x){x})
   
   qEst=NULL
   qTrue=NULL
   for (i in 1:40){
      this.q=qInd[,i]
      this.ID=cut(actual[vpn==i],this.q)
      this.ID[is.na(this.ID)]=levels(this.ID)[1] #smallest number is coded as missing
      qEst=cbind(qEst,tapply(guess[vpn==i],this.ID,mean )) #mean guess for each quintile
      qTrue=cbind(qTrue,tapply(actual[vpn==i],this.ID,mean )) #mean guess for each quintile
   }
   
   #Total underestimate
   diff=qTrue-qEst
   
   #Percentage underestimate
   pUnderestimate=100*(1-qEst/qTrue)
   round(rowMeans(pUnderestimate),1) #mean underestimate within each quintile across individuals

   #add points for quintiles to plot
   quint=rowMeans(qTrue)
   points(quint,rowMeans(qEst), cex=1.4, pch=21, bg="white",col="black" )

   #95% confidence interval across individual means
   qqci95Est=apply(qEst,1, bootstrapCI, conf=.95) 
   arrows(quint, qqci95Est[1,],rowMeans(qTrue), qqci95Est[2,], angle=90, code=3, length=.02 )

   #add text
   atext=round(exp(coefs[1]),2)
   btext=round(coefs[2],2)
   equationtext=bquote(italic(hat(y))==.(atext)%*%italic(y)^.(btext))
   text(150,600, equationtext, cex=1.1)


#----
#Plot bias depending on frequency distribution and number sequence 
   
   #mean for each sequence ("verlauf")
   meanSeq=tapply(bias, list(verlauf), mean)
   
   #bootstrap across individual means
   biasMatrix1=tapply(bias,list(verlauf, vpn), mean )
   ciSeq=apply(biasMatrix1, 1, bootstrapCI)
   
   #mean for each distribution ("verteilung")
   meanDist=tapply(bias, list(verteilung), mean)
   
   #bootstrap across individual means
   biasMatrix2=tapply(bias,list(verteilung, vpn), mean )
   ciDist=apply(biasMatrix2, 1, bootstrapCI)
   
   #glue it together
   bigMean= c(meanDist[6], NA, meanDist[1:5], NA,NA, meanSeq[1:4], NA)
   bigCI  = cbind(ciDist[,6], NA, ciDist[,1:5], NA,NA, ciSeq[,1:4], NA)
   
   
   #some cosmetics for names (capitalize first word)
   names(bigMean)[3] ="Uniform"
   names(bigMean)[4] ="Positive skew"
   names(bigMean)[5] ="Negative skew"
   names(bigMean)[6] ="Unimodal"
   names(bigMean)[7] ="Bimodal"
   names(bigMean)[10]="Increasing"
   names(bigMean)[11]="Decreasing"
   names(bigMean)[12]="Peaked"
   names(bigMean)[13]="U-shaped"
    
   
   #draw the barplot
   windows(6,5)
   par(mar=c(6,2,2,7))
   xlim=range(0,bigCI,bigMean, na.rm=T)
   ypos=barplot(bigMean,xlim=xlim, horiz=T, ylab="", names.arg="", xaxt="n")
   arrows(bigCI[1,], ypos,bigCI[2,],ypos, code=3, length=.02, angle=90)
   axis(1, at=seq(-.12, 0, 0.02))
   mtext(expression(paste("Estimation bias:    ",frac( hat(italic(y))-italic(y),italic(y) ))), side=1, line=4.5)
   abline(v=0)
   
   #add labels
   barLab=names(bigMean)
   barLab[1]=expression(bold(Baseline))
   
   xpos=max(xlim)+0.003
   text(x=xpos, y=ypos, labels=barLab, adj=0, xpd=T)
   text(x=xpos, y=ypos[14], labels=expression(bold(Sequence)), adj=0, xpd=T)
   text(x=xpos, y=ypos[8], labels=expression(bold(Distribution)), adj=0, xpd=T)
   

 
   
   
   
   #plot bias depending on the order of trials within the experiment
   
   qqIdx=rep(NA, times=length(order)) #index of order quintiles
   qq=c(0,15,30,45,60,75)
   for (i in 1:(length(qq)-1)) {
     qqIdx[order<=qq[i+1] & order>qq[i]]=i
     }
     
   mBiasOrder=tapply(bias, list(vpn,qqIdx), mean)
   ciBiasOrder=apply(mBiasOrder, 2, bootstrapCI)
   
   windows(5,5)
   par(mar=c(4.1,6,1,1))
   xpos=barplot(colMeans(mBiasOrder), ylim=1.2*range(ciBiasOrder), xlab="Quintiles")
   mtext(expression(paste("Estimation bias:    ",frac( hat(italic(y))-italic(y),italic(y) ))), side=2, line=3)
   abline(h=0)
   arrows(xpos, ciBiasOrder[1,],xpos,ciBiasOrder[2,], code=3, length=.04, angle=90, lwd=2)
   

#=======================
# Mixed regression model 

#remove baseline condition to ensure that the design is fully-factorial
   isBaseline=factor(as.numeric(verteilung=="baseline"), levels=c(0,1),labels=c("noBaseline","baseline"))

   detach(longFrame)
   longFrame.full=longFrame
   longFrame     =longFrame[isBaseline=="noBaseline",]
   longFrame     =droplevels(longFrame)
   attach(longFrame)

#define mixed model 
#random intercept and slope for each individual (2x40 params)
#random intercept for each verteilung within individuals (5*40=200 params)
#random intercept for each verlauf within individuals (4*40 =160 params)

#set av
av=bias

#Note: To avoid warning messages by lmer, re-scale acutal by dividing it by 100 before entering the equation. 
 actual100=actual/100

#baseline model
lmer0=lmer(av~1+                          (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
summary(lmer0)

#true sum as predictor 
lmer1=lmer(av~actual100+                     (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
summary(lmer1)
anova(lmer0,lmer1)
exp((BIC(lmer0)-BIC(lmer1))/2)

#add order of trials within the experiment
lmer2=lmer(av~actual100+order+             (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
summary(lmer2)
exp((BIC(lmer0)-BIC(lmer2))/2) #BF (lmer2 over lmer0) 
exp((BIC(lmer1)-BIC(lmer2))/2) #BF (lmer2 over lmer1)
anova(lmer1,lmer2)

#add sequencial effects 
lmer3=lmer(av~actual100+order+verlauf +    (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
summary(lmer3)
exp((BIC(lmer2)-BIC(lmer3))/2) #BF (lmer3 over lmer2)
anova(lmer2, lmer3)

#add distribution only (not reported in paper)
lmer3.5=lmer(av~actual100+order+verteilung+(actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
summary(lmer3.5)
exp((BIC(lmer2)-BIC(lmer3.5))/2)

#add sequence and distribution
lmer4=lmer(av~actual100+order+verlauf+verteilung + (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
summary(lmer4)
exp((BIC(lmer3)-BIC(lmer4))/2)
exp((BIC(lmer2)-BIC(lmer4))/2)
exp((BIC(lmer1)-BIC(lmer4))/2)
anova(lmer3, lmer4)
#summary of all models
anova(lmer0,lmer1,lmer2,lmer3,lmer3.5, lmer4)


#----------------------------------
#Extend model with other predictors

#Higher moments:

   #variance
   lmer.var=lmer(av~actual100+order+verlauf+variance + (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.var)
   exp((BIC(lmer3)-BIC(lmer.var))/2)
   
   #kurtosis
   lmer.kurt=lmer(av~actual100+order+verlauf+kurtosis + (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.kurt)
   exp((BIC(lmer3)-BIC(lmer.kurt))/2)

   #skewness
   lmer.skew=lmer(av~actual100+order+verlauf+skew + (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.skew)
   exp((BIC(lmer3)-BIC(lmer.skew))/2)

   
#-----------------------
#Test peak-end heuristic
   lmer.peak=lmer(av~actual100+order+verlauf+peak + (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.peak)
   exp((BIC(lmer3)-BIC(lmer.peak))/2)
   
   lmer.end=lmer(av~actual100+order+verlauf+end+ (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.end)
   exp((BIC(lmer3)-BIC(lmer.end))/2)


#rappen endings (proportion of rappen endings <.50)
   lmer.rappen=lmer(av~actual100+order+pRappen+ (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.rappen)
   exp((BIC(lmer3)-BIC(lmer.rappen))/2)
   
   
#proportion of single digit numbers within a sequence
   lmer.singleDigits=lmer(av~actual100+order+pSingleDigits+ (actual100|vpn) + (1|verteilung:vpn)+(1|verlauf:vpn), REML=F) 
   summary(lmer.singleDigits)
   exp((BIC(lmer3)-BIC(lmer.singleDigits))/2)
   
   

#-------
#clean up
detach(longFrame)
