#step1: Generate estimates from the sum-first and the power-first models respectively
#step2: Fit the simulated choice data with both models and check if the models can be recovered

#==================#
# Field study data #
#==================#

#------------------
#load data from file
shopData=read.csv(file="rawDataField.csv") #participants estimates
load("bonList.RData") #a list with receipts that contain the actual prices of each item and the respective category number

#sum first and power first models
mSumFirst   = function (bonList, params){ params[1]*sapply(bonList, sum)^params[2] }

mPowerFirst = function (bonList, params){ sapply(bonList, function(vec) {sum(params[1]*vec^params[2])})  }   


#Function to add noise to the simulated estimates
addNoise = function(vec, noise){
  lower=vec-1*vec*noise/2
  upper=vec+vec*noise/2
  
  noiseVec=runif(length(vec), min=lower, max=upper)
  if(any(noiseVec<=0)) print("Adding noise led to negative values. Consider adding less noise.")
  return(noiseVec)
  
}

#quadratic loss functions
loss.PowerFirst = function (params, bonList, estimate){
  prediction=mPowerFirst(bonList, params[1:2])
  
  #log-log-likelihood (to account for heteroscedascidity)
  ll=dnorm(log(estimate), mean=log(prediction), sd=params[3])
  loss=-2*sum(log(ll))
  
  
  #likelihood
  #ll=dnorm(estimate, mean=prediction, sd=params[3])
  #loss=-2*sum(log(ll))
  
  return(loss)
}


loss.SumFirst = function (params, bonList, estimate){
  prediction=mSumFirst(bonList, params[1:2])
  
  #log-log-likelihood (to account for heteroscedascidity)
  ll=dnorm(log(estimate), mean=log(prediction), sd=params[3])
  loss=-2*sum(log(ll))
  
  #log-likelihood
  #ll=dnorm(estimate, mean=prediction, sd=params[3])
  #loss=-2*sum(log(ll))
  
  return(loss)
}

#Function to compare fit of both models
returnFit = function(fit1,fit2){
  
  #g2 deviance: smaller loss = better fit
  g2.loss=c(fit1$objective, fit2$objective)
  
  #BIC
  k=length(fit1$par) #number of parameters (same for both models)
  n=length(bonList) #number of observations
  
  BIC1 =fit1$objective+k*log(n)
  BIC2 =fit2$objective+k*log(n)
  
  BF=exp((BIC2-BIC1)/2) #values >1 = evidence for model 1
  
  return(c(g2.diff=g2.loss[2]-g2.loss[1], 
           BICdiff=BIC2-BIC1,
           BF     =BF))
}


noise=0.1 #= 10% noise added to the deterministic model estimates
exponentVec=seq(.8,.98, by=.02)
betaVec=seq(1,1.6, by=.2)
params=expand.grid(noise=noise,exponent=exponentVec,beta=betaVec)

nRounds=dim(params)[1]
resultMatrixSumFirst=resultMatrixPowerFirst=NULL

for (i in 1:nRounds){

  #combination of parameters for this round
  beta=params[i,"beta"]
  exponent=params[i,"exponent"]
  noise=params[i,"noise"]
  
  #------
  # Step 1: Generate deterministic data from both models 
  
  estimateS=mSumFirst(bonList, c(beta,exponent))
  estimateP=mPowerFirst(bonList, c(beta,exponent))


  #------
  #Step 2: Add noise and fit both models to the data
 
  estimateS.noise=addNoise(estimateS,noise)
  estimateP.noise=addNoise(estimateP,noise)
  
  #--------------
  #fit PowerFirst
  fit.PowerFirst.pData=nlminb(start=c(1,1,20), objective=loss.PowerFirst, bonList=bonList, estimate=estimateP.noise, lower=c(0.0001,0.0001, 0.0001))
  fit.PowerFirst.sData=nlminb(start=c(1,1,20), objective=loss.PowerFirst, bonList=bonList, estimate=estimateS.noise, lower=c(0.0001,0.0001, 0.0001))

  #--------------
  #fit SumFirst
  fit.SumFirst.pData=nlminb(start=c(1,1,20), objective=loss.SumFirst, bonList=bonList, estimate=estimateP.noise, lower=c(0.0001,0.0001, 0.0001))
  fit.SumFirst.sData=nlminb(start=c(1,1,20), objective=loss.SumFirst, bonList=bonList, estimate=estimateS.noise, lower=c(0.0001,0.0001, 0.0001))
  
  #keep track of model recovery (positive BIC difference = model could be recovered)
  resultMatrixSumFirst  =rbind(resultMatrixSumFirst  ,c(beta=beta, exponent=exponent,noise=noise,returnFit(fit.SumFirst.sData,fit.PowerFirst.sData)))
  resultMatrixPowerFirst=rbind(resultMatrixPowerFirst,c(beta=beta, exponent=exponent,noise=noise,returnFit(fit.PowerFirst.pData,fit.SumFirst.pData)))
  
  
}
  

#positive value = data-generating model can be distinguished
boxplot(resultMatrixSumFirst[,"BICdiff"], main="Recovery of SumFirst Model", 
        ylab=expression(paste(Delta, "BIC")), col="lightblue")
mtext(paste("min =", round(min(resultMatrixSumFirst[,"BICdiff"]),2)))

boxplot(resultMatrixPowerFirst[,"BICdiff"], main="Recovery of PowerFirst Model", 
        ylab=expression(paste(Delta, "BIC")), col="lightgreen")
mtext(paste("min =", round(min(resultMatrixPowerFirst[,"BICdiff"]),2)))

